@@ -35,6 +35,8 @@ class CLI(object):
       representation
     - collect_activations - For each datapoint, there exists a corresponding
       tensor representation which are collected through this method
+    - hyperopt - Performs an hyper-parameter search
+      with a given strategy and parameters
     """
 
     def __init__(self):
@@ -43,14 +45,20 @@ class CLI(object):
             usage='''ludwig <command> [<args>]
 
 Available sub-commands:
-   experiment            Runs a full experiment training a model and testing it
    train                 Trains a model
    predict               Predicts using a pretrained model
+   evaluate              Evaluate a pretrained model's performance
+   experiment            Runs a full experiment training a model and evaluating it
+   hyperopt              Perform hyperparameter optimization
    serve                 Serves a pretrained model
-   test                  Tests a pretrained model
    visualize             Visualizes experimental results
+   collect_summary       Prints names of weights and layers activations to use with other collect commands
    collect_weights       Collects tensors containing a pretrained model weights
    collect_activations   Collects tensors for each datapoint using a pretrained model
+   export_savedmodel     Exports Ludwig models to SavedModel
+   export_neuropod       Exports Ludwig models to Neuropod
+   preprocess            Preprocess data and saves it into HDF5 and JSON format
+   synthesize_dataset    Creates synthetic data for tesing purposes
 ''')
         parser.add_argument('command', help='Subcommand to run')
         # parse_args defaults to [1:] for args, but you need to
@@ -63,11 +71,6 @@ Available sub-commands:
         # use dispatch pattern to invoke method with same name
         getattr(self, args.command)()
 
-    def experiment(self):
-        from ludwig import experiment
-        ludwig.contrib.contrib_command("experiment", *sys.argv)
-        experiment.cli(sys.argv[2:])
-
     def train(self):
         from ludwig import train
         ludwig.contrib.contrib_command("train", *sys.argv)
@@ -78,21 +81,36 @@ Available sub-commands:
         ludwig.contrib.contrib_command("predict", *sys.argv)
         predict.cli(sys.argv[2:])
 
+    def evaluate(self):
+        from ludwig import evaluate
+        ludwig.contrib.contrib_command("evaluate", *sys.argv)
+        evaluate.cli(sys.argv[2:])
+
+    def experiment(self):
+        from ludwig import experiment
+        ludwig.contrib.contrib_command("experiment", *sys.argv)
+        experiment.cli(sys.argv[2:])
+
+    def hyperopt(self):
+        from ludwig import hyperopt_cli
+        ludwig.contrib.contrib_command("hyperopt", *sys.argv)
+        hyperopt_cli.cli(sys.argv[2:])
+
     def serve(self):
         from ludwig import serve
         ludwig.contrib.contrib_command("serve", *sys.argv)
         serve.cli(sys.argv[2:])
 
-    def test(self):
-        from ludwig import test_performance
-        ludwig.contrib.contrib_command("test", *sys.argv)
-        test_performance.cli(sys.argv[2:])
-
     def visualize(self):
         from ludwig import visualize
         ludwig.contrib.contrib_command("visualize", *sys.argv)
         visualize.cli(sys.argv[2:])
 
+    def collect_summary(self):
+        from ludwig import collect
+        ludwig.contrib.contrib_command('collect_summary', *sys.argv)
+        collect.cli_collect_summary(sys.argv[2:])
+
     def collect_weights(self):
         from ludwig import collect
         ludwig.contrib.contrib_command("collect_weights", *sys.argv)
@@ -103,15 +121,25 @@ Available sub-commands:
         ludwig.contrib.contrib_command("collect_activations", *sys.argv)
         collect.cli_collect_activations(sys.argv[2:])
 
-    def export(self):
+    def export_savedmodel(self):
         from ludwig import export
-        ludwig.contrib.contrib_command("export", *sys.argv)
-        export.cli(sys.argv[2:])
+        ludwig.contrib.contrib_command("export_savedmodel", *sys.argv)
+        export.cli_export_savedmodel(sys.argv[2:])
 
-    def saved_model_predict(self):
+    def export_neuropod(self):
         from ludwig import export
-        ludwig.contrib.contrib_command("saved_model_predict", *sys.argv)
-        export.cli(sys.argv[2:])
+        ludwig.contrib.contrib_command("export_neuropod", *sys.argv)
+        export.cli_export_neuropod(sys.argv[2:])
+
+    def preprocess(self):
+        from ludwig import preprocess
+        ludwig.contrib.contrib_command("preprocess", *sys.argv)
+        preprocess.cli(sys.argv[2:])
+
+    def synthesize_dataset(self):
+        from ludwig.data import dataset_synthesizer
+        ludwig.contrib.contrib_command("synthesize_dataset", *sys.argv)
+        dataset_synthesizer.cli(sys.argv[2:])
 
 
 def main():
