@@ -14,18 +14,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-LUDWIG_VERSION = '0.2.2.6'
+
+LUDWIG_VERSION = '0.3.2'
 
 MODEL_WEIGHTS_FILE_NAME = 'model_weights'
-MODEL_WEIGHTS_PROGRESS_FILE_NAME = 'model_weights_progress'
 MODEL_HYPERPARAMETERS_FILE_NAME = 'model_hyperparameters.json'
-TRAINING_PROGRESS_FILE_NAME = 'training_progress.json'
-TRAIN_SET_METADATA_FILE_NAME = 'train_set_metadata.json'
+TRAIN_SET_METADATA_FILE_NAME = 'training_set_metadata.json'
+TRAINING_PROGRESS_TRACKER_FILE_NAME = 'training_progress.json'
+TRAINING_CHECKPOINTS_DIR_PATH = 'training_checkpoints'
 
 DISABLE_PROGRESSBAR = False
 
-ON_MASTER = True
-
 
 def set_disable_progressbar(value):
     global DISABLE_PROGRESSBAR
@@ -34,23 +33,3 @@ def set_disable_progressbar(value):
 
 def is_progressbar_disabled():
     return DISABLE_PROGRESSBAR
-
-
-def set_on_master(use_horovod):
-    global ON_MASTER
-    if use_horovod:
-        try:
-            import horovod.tensorflow
-            horovod.tensorflow.init()
-            ON_MASTER = horovod.tensorflow.rank() == 0
-        except ImportError:
-            raise ValueError("use_horovod parameter specified, "
-                             "but cannot import horovod.tensorflow. "
-                             "Install horovod following the instructions at: "
-                             " https://github.com/uber/horovod")
-    else:
-        ON_MASTER = True
-
-
-def is_on_master():
-    return ON_MASTER
