@@ -14,18 +14,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import argparse
+import json
 import logging
 import os
 import sys
 import tempfile
 
+import pandas as pd
+
 from ludwig.api import LudwigModel
-from ludwig.contrib import contrib_command
+from ludwig.constants import COLUMN
+from ludwig.contrib import contrib_command, contrib_import
 from ludwig.globals import LUDWIG_VERSION
 from ludwig.utils.print_utils import logging_level_registry, print_ludwig
 
@@ -37,9 +37,11 @@ try:
     from starlette.datastructures import UploadFile
     from starlette.requests import Request
     from starlette.responses import JSONResponse
-except ImportError:
+except ImportError as e:
+    logger.error(e)
     logger.error(
-        ' fastapi and other serving dependencies are not installed. '
+        ' fastapi and other serving dependencies cannot be loaded'
+        'and may have not been installed. '
         'In order to install all serving dependencies run '
         'pip install ludwig[serve]'
     )
@@ -51,11 +53,12 @@ COULD_NOT_RUN_INFERENCE_ERROR = {
     "error": "Unexpected Error: could not run inference on model"}
 
 
+
 def server(model):
     app = FastAPI()
 
     input_features = {
-        f['name'] for f in model.model_definition['input_features']
+        f[COLUMN] for f in model.config['input_features']
     }
 
     @app.get('/')
@@ -72,10 +75,37 @@ def server(model):
                 return JSONResponse(ALL_FEATURES_PRESENT_ERROR,
                                     status_code=400)
             try:
-                resp = model.predict(data_dict=[entry]).to_dict('records')[0]
+                resp, _ = model.predict(
+                    dataset=[entry], data_format=dict
+                )
+                resp = resp.to_dict('records')[0]
                 return JSONResponse(resp)
             except Exception as e:
-                logger.error("Error: {}".format(str(e)))
+                logger.error("Failed to run predict: {}".format(str(e)))
+                return JSONResponse(COULD_NOT_RUN_INFERENCE_ERROR,
+                                    status_code=500)
+        finally:
+            for f in files:
+                os.remove(f.name)
+
+    @app.post('/batch_predict')
+    async def batch_predict(request: Request):
+        form = await request.form()
+        files, data = convert_batch_input(form)
+        data_df = pd.DataFrame.from_records(data['data'],
+                                            index=data.get('index'),
+                                            columns=data['columns'])
+
+        try:
+            if (set(data_df.columns) & input_features) != input_features:
+                return JSONResponse(ALL_FEATURES_PRESENT_ERROR,
+                                    status_code=400)
+            try:
+                resp, _ = model.predict(dataset=data_df)
+                resp = resp.to_dict('split')
+                return JSONResponse(resp)
+            except Exception as e:
+                logger.error("Failed to run batch_predict: {}".format(str(e)))
                 return JSONResponse(COULD_NOT_RUN_INFERENCE_ERROR,
                                     status_code=500)
         finally:
@@ -85,29 +115,65 @@ def server(model):
     return app
 
 
+def _write_file(v, files):
+    # Convert UploadFile to a NamedTemporaryFile to ensure it's on the disk
+    suffix = os.path.splitext(v.filename)[1]
+    named_file = tempfile.NamedTemporaryFile(
+        delete=False, suffix=suffix)
+    files.append(named_file)
+    named_file.write(v.file.read())
+    named_file.close()
+    return named_file.name
+
+
 def convert_input(form):
-    '''
-    Returns a new input and a list of files to be cleaned up
-    '''
+    """Returns a new input and a list of files to be cleaned up"""
     new_input = {}
     files = []
     for k, v in form.multi_items():
         if type(v) == UploadFile:
-            # Convert UploadFile to a NamedTemporaryFile to ensure it's on the disk
-            suffix = os.path.splitext(v.filename)[1]
-            named_file = tempfile.NamedTemporaryFile(
-                delete=False, suffix=suffix)
-            files.append(named_file)
-            named_file.write(v.file.read())
-            named_file.close()
-            new_input[k] = named_file.name
+            new_input[k] = _write_file(v, files)
         else:
             new_input[k] = v
 
-    return (files, new_input)
+    return files, new_input
 
 
-def run_server(model_path, host, port):
+def convert_batch_input(form):
+    """Returns a new input and a list of files to be cleaned up"""
+    files = []
+    file_index = {}
+    for k, v in form.multi_items():
+        if type(v) == UploadFile:
+            file_index[v.filename] = _write_file(v, files)
+
+    data = json.loads(form['dataset'])
+    for row in data['data']:
+        for i in range(len(row)):
+            if row[i] in file_index:
+                row[i] = file_index[row[i]]
+
+    return files, data
+
+
+def run_server(
+        model_path: str,
+        host: str,
+        port: int
+) -> None:
+    """
+    Loads a pre-trained model and serve it on an http server.
+
+    # Inputs
+
+    :param model_path: (str) filepath to pre-trained model.
+    :param host: (str, default: `0.0.0.0`) host ip address for the server to use.
+    :param port: (int, default: `8000`) port number for the server to use.
+
+    # Return
+
+    :return: (`None`)
+    """
     model = LudwigModel.load(model_path)
     app = server(model)
     uvicorn.run(app, host=host, port=port)
@@ -158,8 +224,9 @@ def cli(sys_argv):
 
     args = parser.parse_args(sys_argv)
 
+    args.logging_level = logging_level_registry[args.logging_level]
     logging.getLogger('ludwig').setLevel(
-        logging_level_registry[args.logging_level]
+        args.logging_level
     )
     global logger
     logger = logging.getLogger('ludwig.serve')
@@ -170,5 +237,6 @@ def cli(sys_argv):
 
 
 if __name__ == '__main__':
+    contrib_import()
     contrib_command("serve", *sys.argv)
     cli(sys.argv[1:])
