@@ -14,9 +14,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-from __future__ import absolute_import
-from __future__ import division
-
 import copy
 import logging
 import sys
@@ -24,9 +21,10 @@ from collections import Counter
 from sys import platform
 
 import numpy as np
+import pandas as pd
 
 import ludwig.contrib
-from ludwig.constants import TRAINING, VALIDATION
+from ludwig.constants import TRAINING, TYPE, VALIDATION
 
 logger = logging.getLogger(__name__)
 
@@ -51,6 +49,9 @@ except ImportError:
     )
     sys.exit(-1)
 
+INT_QUANTILES = 10
+FLOAT_QUANTILES = 10
+
 
 # plt.rc('xtick', labelsize='x-large')
 # plt.rc('ytick', labelsize='x-large')
@@ -370,9 +371,10 @@ def donut(
         outside_labels,
         outside_groups,
         title=None,
+        tight_layout=None,
         filename=None
 ):
-    fig, ax = plt.subplots()
+    fig, ax = plt.subplots(figsize=(7,5))
 
     if title is not None:
         ax.set_title(title)
@@ -416,8 +418,9 @@ def donut(
                  pctdistance=1 - (width / 2) / (1 - width),
                  colors=inside_colors, startangle=90, counterclock=False,
                  textprops={'color': 'w', 'weight': 'bold',
-                            'path_effects': [PathEffects.withStroke(linewidth=3,
-                                                                    foreground='black')]})
+                            'path_effects': [
+                                PathEffects.withStroke(linewidth=3,
+                                                       foreground='black')]})
     plt.setp(inside + outside, width=width, edgecolor='white')
 
     wedges = []
@@ -431,11 +434,13 @@ def donut(
             labels.append(outside_labels[so_far])
             so_far += 1
 
-    ax.legend(wedges, labels, frameon=True)
-    plt.tight_layout()
+    if tight_layout:
+        ax.legend(wedges, labels, frameon=True, loc=1, bbox_to_anchor=(1.30, 1.00))
+    else:
+        ax.legend(wedges, labels, frameon=True, loc=1, bbox_to_anchor=(1.50, 1.00))
     ludwig.contrib.contrib_command("visualize_figure", plt.gcf())
     if filename:
-        plt.savefig(filename)
+        plt.savefig(filename, bbox_inches = "tight")
     else:
         plt.show()
 
@@ -706,11 +711,39 @@ def confidence_fitlering_3d_plot(
     handle_1.set_color(colors[0])
     handle_2.set_color(colors[1])
 
-    handle_1._edgecolors2d = handle_1._edgecolors3d
-    handle_2._edgecolors2d = handle_2._edgecolors3d
+    ### the next block is needed because matplotlib 3.3.3 renamed
+    # _edgecolors3d -> _edgecolor3d
+    # _facecolors3d -> _facecolor3d
+    # but we want to try to keep compatibility with older versions
+    ##### BEGIN COMPATIBILITY BLOCK #####
+    if hasattr(handle_1, '_edgecolors3d'):
+        edgecolor3d = handle_1._edgecolors3d
+    else:
+        edgecolor3d = handle_1._edgecolor3d
+    handle_1._edgecolors2d = edgecolor3d
+    handle_1._edgecolor2d = edgecolor3d
 
-    handle_1._facecolors2d = handle_1._facecolors3d
-    handle_2._facecolors2d = handle_2._facecolors3d
+    if hasattr(handle_2, '_edgecolors3d'):
+        edgecolor3d = handle_2._edgecolors3d
+    else:
+        edgecolor3d = handle_2._edgecolor3d
+    handle_2._edgecolors2d = edgecolor3d
+    handle_2._edgecolor2d = edgecolor3d
+
+    if hasattr(handle_1, '_facecolors3d'):
+        facecolor3d = handle_1._facecolors3d
+    else:
+        facecolor3d = handle_1._facecolor3d
+    handle_1._facecolors2d = facecolor3d
+    handle_1._facecolor2d = facecolor3d
+
+    if hasattr(handle_2, '_facecolors3d'):
+        facecolor3d = handle_2._facecolors3d
+    else:
+        facecolor3d = handle_2._facecolor3d
+    handle_2._facecolors2d = facecolor3d
+    handle_2._facecolor2d = facecolor3d
+    ##### END COMPATIBILITY BLOCK #####
 
     ax.legend(frameon=True, loc=3, handles=[handle_1, handle_2])
 
@@ -861,7 +894,6 @@ def calibration_plot(
                         i] if algorithm_names is not None and i < len(
                         algorithm_names) else '')
 
-
     ticks = np.linspace(0.0, 1.0, num=11)
     plt.xlim([-0.05, 1.05])
     plt.xticks(ticks)
@@ -899,7 +931,6 @@ def brier_plot(
     plt.xlabel('class')
     plt.ylabel('brier')
 
-    x = np.array(range(brier_scores.shape[0]))
     for i in range(brier_scores.shape[1]):
         plt.plot(brier_scores[:, i],
                  label=algorithm_names[
@@ -1014,19 +1045,19 @@ def double_axis_line_plot(
         ax1.set_xticklabels(labels, rotation=45, ha='right')
         ax1.set_xticks(np.arange(len(labels)))
 
-    ax1.set_ylabel(y2_name, color=colors[1])
+    ax1.set_ylabel(y1_name, color=colors[1])
     ax1.tick_params('y', colors=colors[1])
-    ax1.set_ylim(min(y2), max(y2))
+    ax1.set_ylim(min(y1_sorted), max(y1_sorted))
 
     ax2 = ax1.twinx()
-    ax2.set_ylabel(y1_name, color=colors[0])
+    ax2.set_ylabel(y2_name, color=colors[0])
     ax2.tick_params('y', colors=colors[0])
-    ax2.set_ylim(min(y1_sorted), max(y1_sorted))
+    ax2.set_ylim(min(y2), max(y2))
 
-    ax1.plot(y2, label=y2_name, color=colors[1],
+    ax1.plot(y1_sorted, label=y1_name, color=colors[1],
+             linewidth=4)
+    ax2.plot(y2, label=y2_name, color=colors[0],
              linewidth=3)
-    ax2.plot(y1_sorted, label=y1_name, color=colors[0],
-             linewidth=4.0)
 
     fig.tight_layout()
     ludwig.contrib.contrib_command("visualize_figure", plt.gcf())
@@ -1177,3 +1208,205 @@ def bar_plot(
         plt.savefig(filename)
     else:
         plt.show()
+
+
+def hyperopt_report(
+        hyperparameters,
+        hyperopt_results_df,
+        metric,
+        filename_template,
+        float_precision=3
+):
+    title = "Hyperopt Report: {}"
+    for hp_name, hp_params in hyperparameters.items():
+        if hp_params[TYPE] == 'int':
+            hyperopt_int_plot(
+                hyperopt_results_df,
+                hp_name,
+                metric,
+                title.format(hp_name),
+                filename_template.format(
+                    hp_name) if filename_template else None
+            )
+        elif hp_params[TYPE] == 'float':
+            hyperopt_float_plot(
+                hyperopt_results_df,
+                hp_name,
+                metric,
+                title.format(hp_name),
+                filename_template.format(
+                    hp_name) if filename_template else None,
+                log_scale_x=hp_params[
+                                'scale'] == 'log' if 'scale' in hp_params else False
+            )
+        elif hp_params[TYPE] == 'category':
+            hyperopt_category_plot(
+                hyperopt_results_df,
+                hp_name,
+                metric,
+                title.format(hp_name),
+                filename_template.format(
+                    hp_name) if filename_template else None
+            )
+
+    # quantize float and int columns
+    for hp_name, hp_params in hyperparameters.items():
+        if hp_params[TYPE] == 'int':
+            num_distinct_values = len(hyperopt_results_df[hp_name].unique())
+            if num_distinct_values > INT_QUANTILES:
+                hyperopt_results_df[hp_name] = pd.qcut(
+                    hyperopt_results_df[hp_name],
+                    q=INT_QUANTILES,
+                    precision=0
+                )
+        elif hp_params[TYPE] == 'float':
+            hyperopt_results_df[hp_name] = pd.qcut(
+                hyperopt_results_df[hp_name],
+                q=FLOAT_QUANTILES,
+                precision=float_precision,
+                duplicates='drop',
+            )
+
+    hyperopt_pair_plot(
+        hyperopt_results_df,
+        metric,
+        title.format("pair plot"),
+        filename_template.format('pair_plot') if filename_template else None
+    )
+
+
+def hyperopt_int_plot(
+        hyperopt_results_df,
+        hp_name,
+        metric,
+        title,
+        filename,
+        log_scale_x=False,
+        log_scale_y=True
+):
+    sns.set_style('whitegrid')
+    plt.figure()
+    seaborn_figure = sns.scatterplot(
+        x=hp_name,
+        y=metric,
+        data=hyperopt_results_df
+    )
+    seaborn_figure.set_title(title)
+    if log_scale_x:
+        seaborn_figure.set(xscale="log")
+    if log_scale_y:
+        seaborn_figure.set(yscale="log")
+    seaborn_figure.xaxis.set_major_locator(ticker.MaxNLocator(integer=True))
+    seaborn_figure.xaxis.set_major_formatter(ticker.ScalarFormatter())
+    seaborn_figure.xaxis.set_minor_formatter(ticker.NullFormatter())
+    seaborn_figure.figure.tight_layout()
+    if filename:
+        seaborn_figure.figure.savefig(filename)
+    else:
+        seaborn_figure.figure.show()
+
+
+def hyperopt_float_plot(
+        hyperopt_results_df,
+        hp_name,
+        metric,
+        title,
+        filename,
+        log_scale_x=False,
+        log_scale_y=True
+):
+    sns.set_style('whitegrid')
+    plt.figure()
+    seaborn_figure = sns.scatterplot(
+        x=hp_name,
+        y=metric,
+        data=hyperopt_results_df
+    )
+    seaborn_figure.set_title(title)
+    seaborn_figure.set(ylabel=metric)
+    if log_scale_x:
+        seaborn_figure.set(xscale="log")
+    if log_scale_y:
+        seaborn_figure.set(yscale="log")
+    seaborn_figure.figure.tight_layout()
+    if filename:
+        seaborn_figure.figure.savefig(filename)
+    else:
+        seaborn_figure.figure.show()
+
+
+def hyperopt_category_plot(
+        hyperopt_results_df,
+        hp_name,
+        metric,
+        title,
+        filename,
+        log_scale=True
+):
+    sns.set_style('whitegrid')
+    plt.figure()
+    seaborn_figure = sns.violinplot(
+        x=hp_name,
+        y=metric,
+        data=hyperopt_results_df,
+        fit_reg=False
+    )
+    seaborn_figure.set_title(title)
+    seaborn_figure.set(ylabel=metric)
+    sns.despine()
+    if log_scale:
+        seaborn_figure.set(yscale="log")
+    plt.tight_layout()
+    if filename:
+        plt.savefig(filename)
+    else:
+        plt.show()
+
+
+def hyperopt_pair_plot(
+        hyperopt_results_df,
+        metric,
+        title,
+        filename
+):
+    params = sorted(list(hyperopt_results_df.keys()))
+    params.remove(metric)
+    num_param = len(params)
+
+    sns.set_style('white')
+    fig = plt.figure(figsize=(20, 20))
+    fig.suptitle(title)
+    gs = fig.add_gridspec(num_param, num_param)
+
+    for i, param1 in enumerate(params):
+        for j, param2 in enumerate(params):
+            if i != j:
+                ax = fig.add_subplot(gs[i, j])
+                heatmap = hyperopt_results_df.pivot_table(
+                    index=param1,
+                    columns=param2,
+                    values=metric,
+                    aggfunc='mean'
+                )
+                sns.heatmap(
+                    heatmap,
+                    linewidths=1,
+                    cmap="viridis",
+                    cbar_kws={'label': metric},
+                    ax=ax,
+                )
+
+    plt.tight_layout(pad=5)
+    if filename:
+        plt.savefig(filename)
+    else:
+        plt.show()
+
+
+def hyperopt_hiplot(
+        hyperopt_df,
+        filename,
+):
+    import hiplot as hip
+    experiment = hip.Experiment.from_dataframe(hyperopt_df)
+    experiment.to_html(filename)
